#!/usr/bin/python
#
# Katello Shell
# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import sys
import codecs

# Change encoding of output streams when no encoding is forced via $PYTHONIOENCODING
# or setting in lib/python{version}/site-packages
if sys.getdefaultencoding() == 'ascii':
    writer_class = codecs.getwriter('utf-8')
    if sys.stdout.encoding == None:
        sys.stdout = writer_class(sys.stdout)
    if sys.stderr.encoding == None:
        sys.stderr = writer_class(sys.stderr)


from katello.client.core import (
  activation_key,
  environment,
  organization,
  user,
  user_role,
  provider,
  ping,
  version,
  product,
  repo,
  packagegroup,
  permission,
  distribution,
  package,
  errata,
  system,
  sync_plan,
  shell_command,
  template,
  changeset,
  client,
  filters,
  gpg_key
)

def setup_admin(admin):
    akey_cmd = activation_key.ActivationKey()
    akey_cmd.add_action('create', activation_key.Create())
    akey_cmd.add_action('info', activation_key.Info())
    akey_cmd.add_action('list', activation_key.List())
    akey_cmd.add_action('update', activation_key.Update())
    akey_cmd.add_action('delete', activation_key.Delete())
    admin.add_command('activation_key', akey_cmd)

    env_cmd = environment.Environment()
    env_cmd.add_action('create', environment.Create())
    env_cmd.add_action('info', environment.Info())
    env_cmd.add_action('list', environment.List())
    env_cmd.add_action('update', environment.Update())
    env_cmd.add_action('delete', environment.Delete())
    admin.add_command('environment', env_cmd)

    org_cmd = organization.Organization()
    org_cmd.add_action('create', organization.Create())
    org_cmd.add_action('info', organization.Info())
    org_cmd.add_action('list', organization.List())
    org_cmd.add_action('update', organization.Update())
    org_cmd.add_action('delete', organization.Delete())
    org_cmd.add_action('uebercert', organization.GenerateDebugCert())
    org_cmd.add_action('subscriptions', organization.ShowSubscriptions())
    admin.add_command('org', org_cmd)

    user_cmd = user.User()
    user_cmd.add_action('create', user.Create())
    user_cmd.add_action('info', user.Info())
    user_cmd.add_action('list', user.List())
    user_cmd.add_action('update', user.Update())
    user_cmd.add_action('delete', user.Delete())
    user_cmd.add_action('report', user.Report())
    user_cmd.add_action('assign_role', user.AssignRole(True))
    user_cmd.add_action('unassign_role', user.AssignRole(False))
    user_cmd.add_action('list_roles', user.ListRoles())
    user_cmd.add_action('sync_ldap_roles', user.SyncLdapRoles())
    admin.add_command('user', user_cmd)

    user_role_cmd = user_role.UserRole()
    user_role_cmd.add_action('create', user_role.Create())
    user_role_cmd.add_action('info', user_role.Info())
    user_role_cmd.add_action('list', user_role.List())
    user_role_cmd.add_action('update', user_role.Update())
    user_role_cmd.add_action('add_ldap_group', user_role.AddLdapGroup())
    user_role_cmd.add_action('remove_ldap_group', user_role.RemoveLdapGroup())
    user_role_cmd.add_action('delete', user_role.Delete())
    admin.add_command('user_role', user_role_cmd)

    permission_cmd = permission.Permission()
    permission_cmd.add_action('create', permission.Create())
    permission_cmd.add_action('list', permission.List())
    permission_cmd.add_action('delete', permission.Delete())
    permission_cmd.add_action('available_verbs', permission.ListAvailableVerbs())
    admin.add_command('permission', permission_cmd)

    admin.add_command('ping', ping.Status())
    admin.add_command('version', version.Info())

    prod_cmd = product.Product()
    prod_cmd.add_action('create', product.Create())
    prod_cmd.add_action('update', product.Update())
    prod_cmd.add_action('list', product.List())
    prod_cmd.add_action('delete', product.Delete())
    prod_cmd.add_action('synchronize', product.Sync())
    prod_cmd.add_action('cancel_sync', product.CancelSync())
    prod_cmd.add_action('status', product.Status())
    prod_cmd.add_action('promote', product.Promote())
    prod_cmd.add_action('list_filters', product.ListFilters())
    prod_cmd.add_action('add_filter', product.AddRemoveFilter(True))
    prod_cmd.add_action('remove_filter', product.AddRemoveFilter(False))
    prod_cmd.add_action('set_plan', product.SetSyncPlan())
    prod_cmd.add_action('remove_plan', product.RemoveSyncPlan())
    admin.add_command('product', prod_cmd)

    repo_cmd = repo.Repo()
    repo_cmd.add_action('create', repo.Create())
    repo_cmd.add_action('update', repo.Update())
    repo_cmd.add_action('discover', repo.Discovery())
    repo_cmd.add_action('info', repo.Info())
    repo_cmd.add_action('list', repo.List())
    repo_cmd.add_action('delete', repo.Delete())
    repo_cmd.add_action('status', repo.Status())
    repo_cmd.add_action('synchronize', repo.Sync())
    repo_cmd.add_action('cancel_sync', repo.CancelSync())
    repo_cmd.add_action('enable', repo.Enable(True))
    repo_cmd.add_action('disable', repo.Enable(False))
    repo_cmd.add_action('list_filters', repo.ListFilters())
    repo_cmd.add_action('add_filter', repo.AddRemoveFilter(True))
    repo_cmd.add_action('remove_filter', repo.AddRemoveFilter(False))

    admin.add_command('repo', repo_cmd)

    package_group_cmd = packagegroup.PackageGroup()
    package_group_cmd.add_action('list', packagegroup.List())
    package_group_cmd.add_action('info', packagegroup.Info())
    package_group_cmd.add_action('category_list', packagegroup.CategoryList())
    package_group_cmd.add_action('category_info', packagegroup.CategoryInfo())
    admin.add_command('package_group',package_group_cmd)

    dist_cmd = distribution.Distribution()
    dist_cmd.add_action('info', distribution.Info())
    dist_cmd.add_action('list', distribution.List())
    admin.add_command('distribution', dist_cmd)

    pack_cmd = package.Package()
    pack_cmd.add_action('info', package.Info())
    pack_cmd.add_action('list', package.List())
    pack_cmd.add_action('search', package.Search())
    admin.add_command('package', pack_cmd)

    errata_cmd = errata.Errata()
    errata_cmd.add_action('list', errata.List())
    errata_cmd.add_action('info', errata.Info())
    errata_cmd.add_action('system', errata.SystemErrata())
    admin.add_command('errata', errata_cmd)

    system_cmd = system.System()
    system_cmd.add_action('list', system.List())
    system_cmd.add_action('register', system.Register())
    system_cmd.add_action('unregister', system.Unregister())
    system_cmd.add_action('subscriptions', system.Subscriptions())
    system_cmd.add_action('subscribe', system.Subscribe())
    system_cmd.add_action('unsubscribe', system.Unsubscribe())
    system_cmd.add_action('info', system.Info())
    system_cmd.add_action('packages', system.InstalledPackages())
    system_cmd.add_action('facts', system.Facts())
    system_cmd.add_action('update', system.Update())
    system_cmd.add_action('report', system.Report())
    system_cmd.add_action('tasks', system.TasksList())
    system_cmd.add_action('task', system.TaskInfo())
    system_cmd.add_action('releases', system.Releases())
    system_cmd.add_action('remove_deletion', system.RemoveDeletion())
    admin.add_command('system', system_cmd)

    sync_plan_cmd = sync_plan.SyncPlan()
    sync_plan_cmd.add_action('create', sync_plan.Create())
    sync_plan_cmd.add_action('info', sync_plan.Info())
    sync_plan_cmd.add_action('list', sync_plan.List())
    sync_plan_cmd.add_action('update', sync_plan.Update())
    sync_plan_cmd.add_action('delete', sync_plan.Delete())
    admin.add_command('sync_plan', sync_plan_cmd)

    template_cmd = template.Template()
    template_cmd.add_action('create', template.Create())
    template_cmd.add_action('import', template.Import())
    template_cmd.add_action('export', template.Export())
    template_cmd.add_action('list', template.List())
    template_cmd.add_action('info', template.Info())
    template_cmd.add_action('update', template.Update())
    template_cmd.add_action('delete', template.Delete())
    admin.add_command('template', template_cmd)

    admin.add_command('shell', shell_command.ShellAction(admin))

    prov_cmd = provider.Provider()
    prov_cmd.add_action('create', provider.Update(create=True))
    prov_cmd.add_action('info', provider.Info())
    prov_cmd.add_action('list', provider.List())
    prov_cmd.add_action('update', provider.Update())
    prov_cmd.add_action('delete', provider.Delete())
    prov_cmd.add_action('synchronize', provider.Sync())
    prov_cmd.add_action('cancel_sync', provider.CancelSync())
    prov_cmd.add_action('status', provider.Status())
    prov_cmd.add_action('import_manifest', provider.ImportManifest())
    prov_cmd.add_action('refresh_products', provider.RefreshProducts())
    admin.add_command('provider', prov_cmd)

    cset_cmd = changeset.Changeset()
    cset_cmd.add_action('create', changeset.Create())
    cset_cmd.add_action('list', changeset.List())
    cset_cmd.add_action('info', changeset.Info())
    cset_cmd.add_action('update', changeset.UpdateContent())
    cset_cmd.add_action('delete', changeset.Delete())
    cset_cmd.add_action('promote', changeset.Promote())
    admin.add_command('changeset', cset_cmd)

    client_cmd = client.Client()
    client_cmd.add_action('remember', client.Remember())
    client_cmd.add_action('forget', client.Forget())
    client_cmd.add_action('saved_options', client.SavedOptions())
    admin.add_command('client', client_cmd)

    filter_cmd = filters.Filter()
    filter_cmd.add_action('create', filters.Create())
    filter_cmd.add_action('list', filters.List())
    filter_cmd.add_action('info', filters.Info())
    filter_cmd.add_action('delete', filters.Delete())
    filter_cmd.add_action('add_package', filters.AddPackage())
    filter_cmd.add_action('remove_package', filters.RemovePackage())
    admin.add_command('filter', filter_cmd)

    gpgkey_cmd = gpg_key.GpgKey()
    gpgkey_cmd.add_action('create', gpg_key.Create())
    gpgkey_cmd.add_action('info', gpg_key.Info())
    gpgkey_cmd.add_action('list', gpg_key.List())
    gpgkey_cmd.add_action('update', gpg_key.Update())
    gpgkey_cmd.add_action('delete', gpg_key.Delete())
    admin.add_command('gpg_key', gpgkey_cmd)
